/*
 * Copyright (C) 2000-2005
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * 
 */
package estella.servidor;
import  estella.lib.*;

import  java.io.FileInputStream;
import  java.io.File;
import  java.net.Socket;
import  java.lang.String;
import  java.util.StringTokenizer;
import  java.io.IOException;
import  java.lang.Thread;
import  java.net.Socket;
import  java.net.SocketTimeoutException;
import  java.io.FileNotFoundException;
import  java.io.IOException;

/**
* Servidor web simples
* @author Cesar Kallas - cesarkallas at gmx . net
* @since 04/12/2005
*/
public class ServidorWeb implements Runnable
{
	private int porta;
	private String repositorio;
	private Log LOG;
	private Constantes CON;
	private Socket sck_cliente;
	private Thread t_conexao;
	private ConexaoServidor servidor;
	
	public ServidorWeb(int porta, String repositorio)
	{
		this.porta = porta;
		this.repositorio = repositorio;
	}
	
	public ServidorWeb(int porta)
	{
		this.porta = porta;
	}
	
	public ServidorWeb()
	{
		porta = 0;
	}
	
  /**
    * Inicia um novo servico de escuta em uma porta
    */
  public void run()
  {
		LOG = new Log();
		CON = new Constantes();
		
		if(repositorio == null)
			repositorio = CON.REPOSITORIO;
		if(porta == 0)
			porta = CON.SERVIDOR_WEB_PORTA;
		
		servidor = new ConexaoServidor(porta);
    servidor.inicia();
    
    // Escuta eternamente requisicao no servidor
    while(true) {
      try {
        sck_cliente = servidor.escuta();
      }
      catch(Exception FalhaEscutarServidor)
      {
        LOG.falha(FalhaEscutarServidor.getMessage());
        return;
      }
      
      // Nova requisicao, ela eh null ? (pode ser quando o cliente conecta e desconecta)
      // Se ok, cria uma nova thread para tratar a requisicao
      if(sck_cliente != null) 
        try {
          t_conexao = new Thread(new trataRequisicaoHTTP(sck_cliente));
          t_conexao.start();
        }
        catch(Exception threadConexao) {
          LOG.falha(threadConexao.getMessage());
          return;
        }
    }
  }
}

/**
* Thread para tratar requisicao recebida pelo servidor
* @param sck_cliente Socket com a conexao do cliente
* @since 23/11/2005
* @author Cesar Kallas - cesarkallas at gmx . net
*/
class trataRequisicaoHTTP implements Runnable
{
  private Conexao conexao;
  private Log LOG;
  private Constantes CON;
	private StringTokenizer requisicao;
	private String nomeArquivo;
	private int indexArquivo;
	private File arquivo;
	private FileInputStream leitorArquivo;
	private byte[] byteArquivo;
		
  public trataRequisicaoHTTP(Socket sck_cliente)
  {
    LOG = new Log();
    CON = new Constantes();
    conexao = new Conexao(sck_cliente);
  }

  public void run()
  {
		conexao.inicia();
		conexao.setTimeout(CON.SOCKET_SERVER_TIMEOUT);
		requisicao = new StringTokenizer(conexao.recebe());
		if(requisicao.nextToken().equals("GET")) {
			requisicao.nextToken("/");
			requisicao.nextToken("/");
			try {
				indexArquivo = Integer.parseInt(requisicao.nextToken("/"));
			}
			catch(Exception SemNumeroIndex){
			}				
			nomeArquivo = requisicao.nextToken("/");
			try {
				arquivo = new File(nomeArquivo);
				System.out.println(arquivo);
				leitorArquivo = new FileInputStream(arquivo);
				byteArquivo = new byte[(int)arquivo.length()];
				leitorArquivo.read(byteArquivo);
			}
			catch(FileNotFoundException ArquivoNaoExiste) {
				LOG.falha(ArquivoNaoExiste.getMessage());
				conexao.envia("HTTP/1.0 404 Not Found\r\n");
				conexao.fecha();
				return;
			}
			catch(IOException IOArquivo) {
				LOG.falha(IOArquivo.getMessage());
				conexao.envia("HTTP/1.0 400 Bad Request\r\n");
				conexao.fecha();
				return;
			}
			System.out.println("Index: " + indexArquivo + " Arquivo: " + nomeArquivo);
			conexao.envia("HTTP/1.0 200 OK\r\n");
			conexao.envia("Server: Gnutella/0.4\r\n");
			conexao.envia("Content-Type: application/binary\r\n");
			conexao.envia("Content-Length: " + arquivo.length() + "\r\n");
			conexao.envia("\r\n");
	    conexao.envia(byteArquivo);
	    conexao.fecha();
		}
		else {
			LOG.falha("HTTP/1.0 400 Bad Request\r\n");
			conexao.envia("HTTP/1.0 400 Bad Request\r\n");
			conexao.fecha();
		}
  }
}
